#include <ctype.h>
#include <qapp.h>
#include <qpushbt.h>
#include <qcursor.h>
#include <qlcdnum.h>
#include <qfont.h>
#include <qrect.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <qbitmap.h>
#include <qbttngrp.h>
#include <qmsgbox.h>
#include <qdialog.h>
#include <qradiobt.h>
#include <qlined.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "bitmaps/bitmaps.h"
#include "chess.h"
#include "rules.h"
#include "connect.h"

#define BUTW 120
#define BUTH 25

#define NOGAME 0
#define WAITFORMOVE 1
#define HAVETOMOVE 2
#define WAITFORMYNEWGAME 4
#define WAITFOROPNEWGAME 8
#define WAITFORNEWGAME (WAITFORMYNEWGAME | WAITFOROPNEWGAME)
#define RUNNING (WAITFORMOVE | HAVETOMOVE)

#define OPDRAW 1
#define MYDRAW 2
#define DRAWAGREED (OPDRAW|MYDRAW)

#define BROAD 64
int BREITE=BROAD,RAND=20,dboard[8][8][2];
int mycolor,whiteside=0,server,drawstate=0,newgamepending;
char addr[200];
BITMAPTYPE BITMAPS=BITMAPS64;
uchar empty[800];
extern int board[8][8],specialmove,status,mcount;
extern const int BOARD[8][8];
extern "C" void move(FIELD *f1,FIELD *f2);   

char getfigurec(FIELD *f){
 switch (board[f->x][f->y] & PIECES) {
  case PAWN  :return '\0';
  case KNIGHT:return 'N';
  case BISHOP:return 'B';
  case ROOK  :return 'R';
  case QUEEN :return 'Q';
  case KING  :return 'K';
 } 
}

void getmovestring(char *c,FIELD *f1,FIELD *f2){
 char cc;
 sprintf(c,"%d. ",(mcount-1)/2+1);
 if (mycolor==WHITEPIECE) strcat(c,"... ");
 switch (specialmove){
  case SHORTCASTLE:strcat(c,"O-O");break;
  case LONGCASTLE :strcat(c,"O-O-0");break;
  default :
   cc=getfigurec(f1);
   strncat(c,&cc,1);
   cc='a'+f1->x;
   strncat(c,&cc,1);
   cc='1'+f1->y;
   strncat(c,&cc,1);
   if (board[f2->x][f2->y] || (specialmove==ENPASSENT))
   cc=':'; else cc='-';
   strncat(c,&cc,1);
   cc='a'+f2->x;
   strncat(c,&cc,1);
   cc='1'+f2->y;
   strncat(c,&cc,1);
 }  
}


void MyWidget::initboard(){
 int i,j;
 drawstate=0;status=0;mcount=0;whiteside=(!whiteside);
 if (socke->getsock()){
  resign->setEnabled(whiteside);
  draw->setEnabled(TRUE);
  gamestate= (whiteside) ? HAVETOMOVE : WAITFORMOVE;
 }
 newgm->setEnabled(FALSE); 
 for (i=0;i<8;i++)
  for (j=0;j<8;j++){
   board[i][j]=BOARD[j][i];
   if (whiteside){
    dboard[i][j][0]=RAND+i*BREITE;
    dboard[i][j][1]=RAND+(7-j)*BREITE;
   } else {
    dboard[i][j][0]=RAND+(7-i)*BREITE;
    dboard[i][j][1]=RAND+j*BREITE;
   }
 }
 mycolor= WHITEPIECE;
 mylcd->reset(playtime);
 oplcd->reset(playtime);
 if (sc[0]->value() || sc[1]->value()) syst_mes("-------");
 repaint();
}

void getfield (int *x,int *y,FIELD *f){
  QRect rec(RAND,RAND,RAND+8*BREITE,RAND+8*BREITE);
  QPoint po(*x,*y);
  if (rec.contains(po,TRUE)){
//  if ((*x>RAND) && (*x<RAND+8*BREITE) && (*y>RAND) && (*y<RAND+8*BREITE)){
   *x-=RAND;*x/=BREITE;*y-=RAND;*y/=BREITE;
   if (whiteside) { f->x=*x;f->y=7-*y;} else {f->x=7-*x;f->y=*y;}
  } else f->x=-1;
}
void PromoWindow::knight(){
 done(2);
}
void PromoWindow::bishop(){
 done(4);
}
void PromoWindow::rook(){
 done(8);
}
void PromoWindow::queen(){
 done(16);
}

PromoWindow ::PromoWindow(int color, QWidget *parent=0, const char *name="Promotion")
            :QDialog (parent,name,TRUE){
 int i;
 setFixedSize(4*(BREITE),BREITE);
 setCaption("Promotion");
 for (i=0;i<4;i++){
  but[i]=new QPushButton(this);
  if (color & BLACKPIECE) memset(empty,0xff,800);else memset(empty,0,800);
  QBitmap bm(BREITE,BREITE,BITMAPS[0][i+1],TRUE);
  QBitmap bm2(BREITE,BREITE,empty,TRUE);
  QPixmap pm(bm2);
  pm.setMask(bm);
  but[i]->setPixmap(pm);
  but[i]->setGeometry(i*(BREITE),0,BREITE,BREITE);
 }
 connect( but[0], SIGNAL(clicked()),SLOT(knight()));
 connect( but[1], SIGNAL(clicked()),SLOT(bishop()));
 connect( but[2], SIGNAL(clicked()),SLOT(rook()));
 connect( but[3], SIGNAL(clicked()),SLOT(queen()));
}


OptDial::OptDial(int *playtime,QWidget* parent,const char* name)
        :QDialog(parent,name,TRUE){
 p=playtime;
 setFixedSize(300,300);
 setCaption("Connection Parameters");
 bg=new QButtonGroup("I will be",this,"radio");
 bg->setGeometry(10,10,160,160);
 rb1=new QRadioButton ("Server",bg);
 rb1->setGeometry(10,30,80,20);
 rb2=new QRadioButton ("Client connecting to:",bg);
 rb2->setGeometry(10,60,140,95);
 ok=new QPushButton("Ok",this);
 ok->setDefault(TRUE);
 ok->setGeometry(200,250,70,20);
 quit=new QPushButton("Cancel",this);
 quit->setGeometry(100,250,70,20);
 rb1->setChecked(TRUE);
 le=new QLineEdit(rb2);
 le->setGeometry(20,55,110,20);
 le->setEnabled(FALSE);
 le->setText("laura");
 sprintf(txt,"%d",*p/60);
 letime=new QLineEdit(rb2);
 letime->setGeometry(20,75,30,20);
 letime->setText(txt);
 letime->setEnabled(FALSE);
 min=new QLabel(rb2);
 min->setGeometry(55,75,80,20);
 min->setText("min playtime");
 connect (quit,SIGNAL(clicked()),qApp,SLOT(quit()));
 connect (ok,SIGNAL(clicked()),SLOT(accept()));
 connect (le,SIGNAL(returnPressed()),SLOT(accept()));
 connect (letime,SIGNAL(returnPressed()),SLOT(accept()));
 connect (bg,SIGNAL(pressed(int)),SLOT(beclient(int)));
 connect (letime,SIGNAL(textChanged(const char*)),SLOT(chktime(const char*)));
}
void OptDial::beclient(int which){
 le->setEnabled(which);
 letime->setEnabled(which);
 if (which) {le->setFocus();le->selectAll();}
}

void OptDial::chktime(const char *t){
 if (strlen(t)>2) {letime->setText(txt);return;}
 for (int i=0;i<strlen(t);i++)
  if (!isdigit(t[i])) {letime->setText(txt);return;}
 strcpy(txt,t);
 *p=atoi(t)*60;
}

void MyWidget::drawboard(bool all=true){
 int i,j;
 QPainter paint(this);
 char c[2]={0,0};
 paint.setFont( QFont( "Times", 12, QFont::Normal ) );
 if (all){
  for (i=0;i<8;i++)
   for (j=0;j<8;j++){
    if ((i%2)^(j%2)) brush=&blackbrush;else brush=&whitebrush;
    paint.fillRect(RAND+i*BREITE,RAND+j*BREITE,BREITE,BREITE,*brush);
  }
 }
 if (whiteside) c[0]='A';else c[0]='H';
 for (i=RAND;i<8*BREITE+RAND;i+=BREITE){
  paint.drawLine(i,RAND,i,RAND+8*BREITE);
  paint.drawText(i+BREITE/2-3,8*BREITE+12+RAND,c);
  if (whiteside) c[0]++;else c[0]--;
 }
 paint.drawLine(i,RAND,i,RAND+8*BREITE);
 if (whiteside) c[0]='8';else c[0]='1';
 for (i=RAND;i<8*BREITE+RAND;i+=BREITE){
  paint.drawLine(RAND,i,RAND+8*BREITE,i);
  paint.drawText(RAND/2-3,i+BREITE/2+6,c);
  if (whiteside) c[0]--;else c[0]++;
 }
 paint.drawLine(RAND,i,RAND+8*BREITE,i);
}

void MyWidget::drawfield(FIELD *f,bool malen=TRUE){
 int i;
 QPainter paint(this);
 if (!(f->x%2)^(f->y%2)) brush=&blackbrush;else brush=&whitebrush;
 paint.fillRect(dboard[f->x][f->y][0]+1,dboard[f->x][f->y][1]+1,BREITE-1,BREITE-1,*brush);
 if (malen){
  for (i=0;i<6;i++)
   if ((board[f->x][f->y]) & (1<<i)) break;
  if (i==6) return;
  QBitmap bm1(BREITE,BREITE,BITMAPS[0][i],TRUE);
  QBitmap bm2(BREITE,BREITE,BITMAPS[1][i],TRUE);
  QPixmap pm(bm2);
  pm.setMask(bm1);
  pm.fill();
  if (board[f->x][f->y] & WHITEPIECE) paint.setPen(white);
   else paint.setPen(black);
  paint.drawPixmap(dboard[f->x][f->y][0],dboard[f->x][f->y][1],pm);
 }
}

void MyWidget::paintEvent(QPaintEvent *e){
 int i,j;
 FIELD f;
 drawboard(false);
 int *db;
 for (f.x=0;f.x<8;f.x++)
  for (f.y=0;f.y<8;f.y++){
   db=dboard[f.x][f.y];
   if (e->rect().intersects(QRect (db[0],db[1],db[0]+BREITE,db[1]+BREITE))) drawfield(&f,board[f.x][f.y]);
  } 
}

void MyWidget::mousePressEvent(QMouseEvent *e){
 int x,y,figur,i;
 static FIELD f;
 
 
 touched_piece=0;
 x=e->x();y=e->y();
 if (e->button() & RightButton) {iconify();return;}
 if (gamestate!=HAVETOMOVE) return;
 if (e->button() & LeftButton) {
  getfield(&x,&y,&f);
  if (f.x==-1) return;
  if (!(figur=board[f.x][f.y])) return; 
  if (!(figur & mycolor)) return;
  touched_piece=figur;
  touched_field=&f;
//new
  for (i=0;i<6;i++)
   if ((board[f.x][f.y]) & (1<<i)) break;
  if (i==6) return;
  QBitmap bm1(BREITE,BREITE,BITMAPS[0][i],TRUE);
//  QBitmap bm2(BREITE,BREITE,BITMAPS[1][i],TRUE);
  if (figur & BLACKPIECE) memset(empty,0xff,800);else memset(empty,0,800);
  QBitmap bm2(BREITE,BREITE,empty,TRUE);
  drawfield(touched_field,FALSE);
  curs=new QCursor(bm2,bm1);
  setCursor(*curs);
//end
 }
}
void MyWidget::mouseReleaseEvent(QMouseEvent *e){
 int x,y,figur;
 char newmove[50];
 FIELD f,ff[2];
 x=e->x();y=e->y();
 if (e->button() & LeftButton) {
  if (!touched_piece) return;
  setCursor(arrowCursor);
  delete curs;  
  getfield(&x,&y,&f);
  if (f.x==-1) {drawfield(touched_field,TRUE);return;}
  if ((f.x==touched_field->x) && (f.y==touched_field->y)) {drawfield(touched_field,TRUE);return;}
  if (!islegal(touched_field,&f,MYTURN)) {drawfield(touched_field,TRUE);return;}
  ff[0].x=touched_field->x;
  ff[0].y=touched_field->y;
  ff[1].x=f.x;
  ff[1].y=f.y;
  displaymove(ff);
  board[f.x][f.y]=touched_piece;
  drawfield(touched_field,FALSE);
  drawfield(&f,FALSE);
  drawfield(&f);
  ::move(touched_field,&f);
  if (specialmove & (CASTLE|ENPASSENT)) special(touched_field,&f);
  if (specialmove & PROMOTE) promote(&f,TRUE);
  socke->outmove[0]='&';
  translate(ff,socke->outmove+1,0);
  socke->sendmove(socke->outmove);
  drawstate=0;
  oplcd->start();
  mylcd->stop();
  draw->setEnabled(TRUE);
  resign->setEnabled(FALSE);
  strcpy(newmove,ml->text());
  int res=is_end();
  switch  (res) {
   case MATE: strcat(newmove," {check mate}");newgame(1);break;
   case PATT: strcat(newmove," {stalemate}");newgame(0);break;
   default  : gamestate = WAITFORMOVE;break;
  }
  if (status==CHECK) strcat(newmove,"+");
  ml->setText(newmove);
 }
}

void MyWidget::special(FIELD *f1,FIELD *f2){
 FIELD f;

 switch (specialmove){
  case ENPASSENT  :f.x=f2->x;f.y=f1->y;drawfield(&f,FALSE);break;
  case SHORTCASTLE:f.x=f1->x+1;f.y=f1->y;drawfield(&f,TRUE);
                   f.x=7;f.y=f1->y;drawfield(&f,FALSE);break;
  case LONGCASTLE :f.x=f1->x-1;f.y=f1->y;drawfield(&f,TRUE);
                   f.x=0;f.y=f1->y;drawfield(&f,FALSE);break;
 }
}

void MyWidget::displaymove(FIELD *f){
 char mstr[20];
 getmovestring(mstr,f,f+1);
 ml->setText(mstr);
} 

void MyWidget::promote(FIELD *f,bool active){
 PromoWindow *pw;
 if (active) {
  pw=new PromoWindow(board[f->x][f->y] & COLORS,this);
  specialmove|=pw->exec();
 }
 board[f->x][f->y]=(board[f->x][f->y] & COLORS) |((specialmove ^ PROMOTE) & PIECES);
 drawfield(f,FALSE);
 drawfield(f);
}

class Mylbi:public QListBoxText{
public:
   Mylbi(const char*);
   void paint(QPainter *);
};

Mylbi::Mylbi(const char *m)
     :QListBoxText(m){
}
void Mylbi::paint(QPainter *p){
 p->setPen(red);
 QListBoxText::paint(p);
}

void MyWidget::syst_mes(const char *m){
 Mylbi *lbi;
 uint i;
 lbi=new Mylbi(m);
 lb->insertItem(m);
 i=lb->count();
 lb->setTopItem(i); 
 lb->changeItem(lbi,i-1); 
}


MyWidget::MyWidget( QWidget *parent, const char *name )
        : QWidget( parent, name )
{
    mycolor=WHITEPIECE;
    playtime=600;
//    setGeometry( 100, 100, 200, 200 );
    setFixedSize(800 , 600 );
    setCaption("rgNetchess: not connected");
    newgm = new QPushButton( "New Game", this, "ngame" );
    newgm->setGeometry( (8*BREITE-4*BUTW)/2+RAND, 2*RAND+RAND/2+8*BREITE, BUTW, BUTH );
    newgm->setFont( QFont( "Times", 12, QFont::Bold ) );
    newgm->setEnabled(FALSE);
    draw = new QPushButton( "Offer/Accept Draw", this, "draw" );
    draw->setGeometry( newgm->x()+BUTW, newgm->y(), BUTW, BUTH );
    draw->setFont( QFont( "Times", 12, QFont::Bold ) );
    draw->setEnabled(FALSE);
    resign = new QPushButton( "Resign", this, "resign" );
    resign->setGeometry( draw->x()+BUTW, draw->y(), BUTW, BUTH );
    resign->setFont( QFont( "Times", 12, QFont::Bold ) );
    resign->setEnabled(FALSE);
    quit = new QPushButton( "Quit", this, "quit" );
    quit->setGeometry( resign->x()+BUTW, draw->y(), BUTW, BUTH );
    quit->setFont( QFont( "Times", 12, QFont::Bold ) );
    connect(resign,SIGNAL(clicked()),SLOT(res_pressed()));
    connect(draw,SIGNAL(clicked()),SLOT(draw_pressed()));
    connect(newgm,SIGNAL(clicked()),SLOT(newgm_pressed()));
    lb=new QListBox(this);
    lb->setGeometry(BREITE*8+2*RAND,180,238,200);
    le=new QLineEdit(this);
    le->setGeometry(BREITE*8+2*RAND,380,238,20);
    ml=new QLabel(this);
    ml->setGeometry(BREITE*8+2*RAND,160,238,20);
    whitebrush=yellow.dark(125);
    blackbrush=red.dark(112);
    connect( quit, SIGNAL(clicked()), qApp, SLOT(quit()) );
    for (int i=0;i<2;i++){
     sc[i]=new QLCDNumber(4,this);
     sc[i]->setSmallDecimalPoint(TRUE);
     sc[i]->setSegmentStyle(QLCDNumber::Flat);
     sc[i]->display(0.0);
    } 

    OptDial o(&playtime);
    if (!o.exec()) {qApp->quit();exit(-1);return;}
    if (o.rb1->isChecked()) server=1;else {server=0;whiteside=(!whiteside);strcpy(addr,o.le->text());}
    if (server)  socke=new Socke;else socke=new Socke(addr,this);
    mylcd  = new Uhr( playtime, this,"mytime");
    oplcd  = new Uhr( playtime, this,"optime");
    oplcd->setGeometry(BREITE*8+2*RAND,RAND,140,40 );
    mylcd->setGeometry(BREITE*8+2*RAND,RAND+8*BREITE-oplcd->height(),oplcd->width(),oplcd->height());
    sc[0]->setGeometry(oplcd->x()+oplcd->width(),RAND,80,oplcd->height());
    sc[1]->setGeometry(mylcd->x()+mylcd->width(),mylcd->y(),sc[0]->width(),mylcd->height());
    initboard();
    connect(socke,SIGNAL(connected()),SLOT(amconnected()));
    connect(socke,SIGNAL(msg(int)),SLOT(receive(int)));
    if (!server) {
     socke->sendconn();
     socke->outmove[0]='/';
     char pt[5];
     sprintf(pt,"%d",playtime);
     strcpy(socke->outmove+1,pt);
     socke->sendmove(socke->outmove);
    } 
    
}

void MyWidget::amconnected(){
 char ca[200];
 sprintf(ca,"rgNetchess: Connected to %s",socke->conn_name);
 setCaption(ca);
 connect(quit,SIGNAL(clicked()),socke,SLOT(close_socket()));
 connect(le,SIGNAL(returnPressed()),SLOT(sendchat()));
 draw->setEnabled(TRUE);
 resign->setEnabled(whiteside);
 gamestate = (server)? HAVETOMOVE:WAITFORMOVE;
 if (server) {
  connect(oplcd,SIGNAL(over(int)),SLOT(timeover(int)));
  connect(mylcd,SIGNAL(over(int)),SLOT(timeover(int)));
  qApp->beep();
 } 
}

void MyWidget::timeover(int who){
 newgame(who);
 strcpy(socke->outmove,"(0");
 socke->outmove[1]=who;
 socke->sendmove(socke->outmove);
}
void MyWidget::sendchat(){
 if (!strcmp(le->text(),"")) return;
 char echo[MAXCHAT+2]="->";
 socke->outmove[0]='@';
 strcpy(socke->outmove+1,le->text());
 strcpy(echo+2,le->text());
 le->setText("");
 lb->insertItem(echo);
 lb->setTopItem(lb->count());
 socke->sendmove(socke->outmove);
}

void MyWidget::translate(FIELD *f,char *m,int source){
 char mes[3]={0,0,0},promopiece[2]={0,0};
 int i;
 i=0;
 if (source){
  for (int j=0;j<2;j++){
   strncpy(mes,m+2*j,2);
   i=atoi(mes)-1;
   f[j].x=i%8;
   f[j].y=7-i/8;
  }
 } else {
  for (int j=0;j<2;j++){
   i=(7-f[j].y)*8+f[j].x+1;
   sprintf(m+2*j,"%02d",i);
  }
  if (specialmove & PROMOTE) {
   i=board[f[1].x][f[1].y];
   promopiece[0]=(i^(i&COLORS));
   strcat(m,promopiece);
  } 
 } 
}

void MyWidget::receive(int n){
FIELD f[2];
char newmove[50];
 if (!n){
  QMessageBox x;
  x.setCaption("What a pity");
  x.setText("Your opponent has left!");
  x.setButtonText("Quit!");
  x.show();
  socke->close_socket();
  qApp->quit();
 }
 switch (socke->inmove[0]) {
  case '&':if (!(gamestate & WAITFORMOVE)) 
            {syst_mes("unexpected move received");break;}
            translate(f,socke->inmove+1,1);
            if (islegal(f,f+1,1)) {
             displaymove(f);
             qApp->beep();
             ::move(f,f+1);
             drawfield(f,FALSE);
             drawfield(f+1,FALSE);
             drawfield(f+1,TRUE);
             if (specialmove & (CASTLE|ENPASSENT)) special(f,f+1);
             if (specialmove & PROMOTE) {
              specialmove|=socke->inmove[5];
              promote(f+1,FALSE);
             }
             drawstate=0;
             resign->setEnabled(TRUE);
             mylcd->start();
             oplcd->stop();
             strcpy(newmove,ml->text());
             int res=is_end();
             switch  (res) {
               case MATE: strcat(newmove," {check mate}");newgame(2);break;
               case PATT: strcat(newmove," {stalemate}");newgame(0);break;
               default  : gamestate = HAVETOMOVE;break;
             }
             if (status==CHECK) strcat(newmove,"+");
             ml->setText(newmove);
             draw->setEnabled(TRUE);
            }break;
  case '@':lb->insertItem(socke->inmove+1);lb->setTopItem(lb->count());break;
  case '%':switch (socke->inmove[1]){
            case '0':status=RESIGN;newgame(1);break;
            case '1':if (drawstate!=MYDRAW) syst_mes("opponent offers draw");
                     drawstate|=OPDRAW;
                     if (drawstate==DRAWAGREED) newgame(0);break;
            default :syst_mes("unknown system message sent");break;
           };break;
  case '$':switch (gamestate){
           case WAITFORMOVE:
           case HAVETOMOVE :syst_mes("illegal \"new game\" received");break;
           case WAITFORNEWGAME:syst_mes("opponent wants new game");gamestate=WAITFORMYNEWGAME;break;
           case WAITFOROPNEWGAME:gamestate=0;initboard();break;
           default:char mm[100];sprintf(mm,"new game in gamestate %d received",gamestate);
                   syst_mes(mm);break;
           };break;        
  case '/':if (!server) break;
     						{int nt=atoi(socke->inmove+1);
           playtime=nt;
           oplcd->reset(nt);
           mylcd->reset(nt);
           break;}
  case '(':if (server) break;
           newgame((!(socke->inmove[1]-1))+1);break;
  default:syst_mes("unknown data sent");
 } 
 socke->readnot->setEnabled(TRUE);
}

void MyWidget::newgame(int who){
 oplcd->stop();
 mylcd->stop();
 char m[100]="Game over: ",m1[50];
 switch (who) {
  case 0:strcpy(m1,"drawed");for (int i=0;i<2;i++) sc[i]->display(sc[i]->value()+0.5);break;
  case 1:strcpy(m1,"I win");sc[1]->display(sc[1]->value()+1);break;
  case 2:strcpy(m1,"Opponent wins");sc[0]->display(sc[0]->value()+1);break;
 } 
 qApp->beep();
 strcat(m,m1);
 syst_mes(m);
 gamestate=WAITFORNEWGAME;
 newgm->setEnabled(TRUE);
 draw->setEnabled(FALSE);
 resign->setEnabled(FALSE);
}

void MyWidget::res_pressed(){
 strcpy(socke->outmove,"%0");
 socke->sendmove(socke->outmove);
 newgame(2);
}
void MyWidget::draw_pressed(){
 strcpy(socke->outmove,"%1");
 socke->sendmove(socke->outmove);
 if (!drawstate) syst_mes("draw offer sent");
 draw->setEnabled(FALSE);
 if (drawstate & OPDRAW) {newgame(0);return;}
 if (drawstate & MYDRAW) return;
 drawstate=MYDRAW;
}

void MyWidget::newgm_pressed(){
 gamestate^=WAITFORMYNEWGAME;
 newgm->setEnabled(FALSE);
 if (!gamestate) initboard(); else syst_mes("Wait for Opponent");
 socke->outmove[0]='$';
 socke->outmove[1]=0;
 socke->sendmove(socke->outmove);
}

int main( int argc, char **argv ){
    QApplication a( argc, argv );
    MyWidget w;

    a.setMainWidget( &w );
    w.show();
    return a.exec();
}
